/*************************************************************************
*  PrimeTime++                                                           * 
*  This C++ program is a fun little math program that combines two math  *
*  programs into one. It will either prompt the user for a number, then  *
*  displays all prime numbers up to the number that the user chose.  	 *
*  You also have the choice to calculate a number as a prime factor		 *  
*  and display all of its divisors on screen.							 *
*  Written by Chad Jordan                                                *
*  3/17/2007                                                             * 
*************************************************************************/
#include <iostream>
#include <cstdlib>
#include <vector>
#include <cmath>
#include <limits>

using std::cout;
using std::cin;
using std::endl;

void programOne();
void programTwo();
void ClearMultiple(unsigned long nElim, unsigned long intArray[], unsigned long sizeOfArray);

int main() {
    int choice;

    // Main menu
    do {
        cout << "Welcome to PrimeTime++\n" << "**********************\n" << "Select a program to run:\n";
        cout << "1. Prime List\n" << "2. Prime Factors\n" << "0. Exit\n";
        cout << "Enter your choice: ";
        cin >> choice;

        // Execute the appropriate program
        switch (choice) {
            case 1:
                programOne();
                break;
            case 2:
                programTwo();
                break;
            case 0:
                cout << "Exiting...\n";
                break;
            default:
                cout << "Invalid choice. Please choose 0, 1, or 2.\n";
        }
    } while (choice != 0);

    return 0;
}

// Implementation of Program One
void programOne() {
    cout << "\nRunning Program One...\n";
    
    int num = 0;
    cout << "Enter the largest number to list primes up to: ";
    cin >> num;
	bool validInput = false;
	// Input validation loop
    while (!validInput) {
        cout << "Enter the largest number to list primes up to: ";
        cin >> num;

        if (cin.fail() || num < 2) {
            cout << "Invalid input. Please enter a number greater than or equal to 2.\n";

            // Clear the error state and discard invalid input
            cin.clear(); 
            cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n'); 
        } else {
            validInput = true;
        }
    }
    bool isPrime = true;
    for (int i = 2; i <= num; i++) {
        isPrime = true;
        // Optimization: Check up to the square root of `i`
        for (int j = 2; j <= std::sqrt(i); j++) {
            if (i % j == 0) {
                isPrime = false;
                break;
            }
        }
        if (isPrime) {
            cout << "Prime: " << i << endl;
        }
    }
    cout << "Program One finished.\n\n";
    system("PAUSE");
}

void programTwo() {
    cout << "\nRunning Program Two...\n";

    unsigned long nLimit;
    bool bFlag = false;

    while (!bFlag) {
        cout << "Enter a number to calculate its prime factors (2 to 478939): ";
        cin >> nLimit;
         // Check for invalid input
        if (cin.fail() || nLimit < 2 || nLimit > 478939) {
            cout << "Invalid input. Please enter a number between 2 and 478939.\n";
            cin.clear(); 
            cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n'); 
        } else {
            bFlag = true;
        }
    }

    unsigned long inputValues[nLimit];
    for (unsigned long i = 0; i < nLimit; i++) {
        inputValues[i] = i;
    }

    for (unsigned long i = 2; i < nLimit; i++) {
        if (inputValues[i] == i) {
            ClearMultiple(i, inputValues, nLimit);
        }
    }

    cout << nLimit << " is divisible by the following prime numbers:\n";
    unsigned long nTemp = nLimit;
    unsigned long factorCount = 0;
    std::vector<unsigned long> primeFactors;

    // Collect prime factors and count their multiplicities
    for (unsigned long i = 2; i < nLimit; i++) {
        if (inputValues[i] == i) {
            while (nTemp % i == 0) {
                primeFactors.push_back(i);
                nTemp /= i;
                factorCount++;
            }
        }
    }

    if (primeFactors.empty()) {
        cout << nLimit << " is a prime number.\n\n";
    } else {
        // Print prime factors and their multiplication
        cout << nLimit << " = ";
        for (size_t i = 0; i < primeFactors.size(); ++i) {
            cout << primeFactors[i];
            if (i < primeFactors.size() - 1) {
                cout << " * ";
            }
        }
        cout << endl;
    }

    cout << "\nProgram Two finished.\n\n";
    system("PAUSE");
}

// Eliminate multiples of a number from the array
void ClearMultiple(unsigned long nElim, unsigned long integerArray[], unsigned long nLimit) {
    for (unsigned long i = 2 * nElim; i < nLimit; i += nElim) {
        integerArray[i] = 0;
    }
}
